@@ -24,12 +24,13 @@ import os
 import sys
 import traceback
 
+import asyncio
 import aiohttp.web
 import typing
 
-from . import formdata
+import ahapi.formdata
 
-__version__ = '0.1.2'
+__version__ = "0.1.5"
 
 
 class Endpoint:
@@ -56,7 +57,7 @@ class SimpleServer:
                 spec.loader.exec_module(m)
                 endpoint_url = os.path.join(dir_relative, endpoint).strip("/")
                 if hasattr(m, "register"):
-                    self.handlers[endpoint_url] = m.__getattribute__("register")(self)
+                    self.handlers[endpoint_url] = m.__getattribute__("register")(self.state)
                     print(f"Registered endpoint /{endpoint_url}")
                 else:
                     print(f"Could not find entry point 'register()' in {endpoint_path}, skipping!")
@@ -64,7 +65,9 @@ class SimpleServer:
                 print(f"Traversing {endpoint_path}")
                 self.load_api_dir(endpoint_path)
 
-    def __init__(self, api_dir: str = "endpoints", bind_ip: str = "127.0.0.1", bind_port: int = 8080, state: typing.Any = None):
+    def __init__(
+        self, api_dir: str = "endpoints", bind_ip: str = "127.0.0.1", bind_port: int = 8080, state: typing.Any = None
+    ):
         print("==== Starting HTTP API server... ====")
         self.state = state
         self.handlers = {}
@@ -76,7 +79,6 @@ class SimpleServer:
         # Load each URL endpoint
         self.load_api_dir(api_dir)
 
-
     async def handle_request(self, request: aiohttp.web.BaseRequest) -> aiohttp.web.Response:
         """Generic handler for all incoming HTTP requests"""
         resp: aiohttp.web.Response
@@ -85,7 +87,7 @@ class SimpleServer:
         headers = {"Server": "ahapi v/%s" % __version__}
 
         # Figure out who is going to handle this request, if any, while allowing path info.
-        handler = '__404'
+        handler = "__404"
         segments = request.path.strip("/").split("/")
         for i in range(0, len(segments)):
             partial_url = "/".join(segments)
@@ -95,13 +97,13 @@ class SimpleServer:
             segments.pop()
 
         body_type = "form"
-        cct = request.headers.get('content-type', 'foobar')
-        if cct.lower() == 'application/json':
+        cct = request.headers.get("content-type", "foobar")
+        if cct.lower() == "application/json":
             body_type = "json"
 
         # Parse form/json data if any
         try:
-            indata = await formdata.parse_formdata(body_type, request)
+            indata = await ahapi.formdata.parse_formdata(body_type, request)
         except ValueError as e:
             return aiohttp.web.Response(headers=headers, status=400, text=str(e))
 
@@ -112,9 +114,15 @@ class SimpleServer:
                 # but could be an exception (that needs a traceback) OR
                 # it could be a custom response, which we just pass along to the client.
                 output = await self.handlers[handler].exec(self.state, request, indata)
-                headers["content-type"] = "application/json"
                 if output and not isinstance(output, aiohttp.web.Response):
-                    jsout = json.dumps(output, indent=2)
+                    if isinstance(output, str):
+                        headers["content-type"] = "text/html"
+                        jsout = output
+                    elif isinstance(output, dict) or isinstance(output, list) or isinstance(output, tuple):
+                        headers["content-type"] = "application/json"
+                        jsout = json.dumps(output, indent=2)
+                    else:
+                        raise ValueError(f"Could not determine output type from API call to {handler}")
                     headers["Content-Length"] = str(len(jsout))
                     return aiohttp.web.Response(headers=headers, status=200, text=jsout)
                 elif isinstance(output, aiohttp.web.Response):
@@ -130,10 +138,14 @@ class SimpleServer:
         else:
             return aiohttp.web.Response(headers=headers, status=404, text="API Endpoint not found!")
 
-    async def loop(self):
+    async def loop(self, forever=True):
         self.server = aiohttp.web.Server(self.handle_request)
         runner = aiohttp.web.ServerRunner(self.server)
         await runner.setup()
         site = aiohttp.web.TCPSite(runner, self.bind_ip, self.bind_port)
         await site.start()
         print("==== HTTP API Server running on %s:%s ====" % (self.bind_ip, self.bind_port))
+        if forever:
+            while True:
+                await asyncio.sleep(100)
+
